@@ -4,6 +4,7 @@ import gzip
 import json
 import logging
 import multiprocessing
+import shutil
 import sys
 from functools import partial
 from pathlib import Path
@@ -16,7 +17,7 @@ import tqdm
 from tqdm.contrib.concurrent import process_map
 from tqdm.contrib.logging import logging_redirect_tqdm
 
-from gpas import misc
+from gpas import misc, __version__
 from gpas.misc import (
     DEFAULT_ENVIRONMENT,
     ENDPOINTS,
@@ -62,6 +63,7 @@ def fetch_user_details(access_token, environment: ENVIRONMENTS):
         logging.debug(f"{result=}")
         user = result.get("userName")
         organisation = result.get("organisation")
+        date_mask = result.get("maskCollectionDate")  # Expects {"N", "MONTH", "WEEK"}
         allowed_tags = [d.get("tagName") for d in result.get("tags", {})]
     except requests.exceptions.HTTPError as e:
         status_code = e.response.status_code
@@ -71,7 +73,18 @@ def fetch_user_details(access_token, environment: ENVIRONMENTS):
             ) from None
         else:
             raise e from None
-    return user, organisation, allowed_tags
+    # try:
+    #     r = requests.get(
+    #         "https://api.github.com/repos/GlobalPathogenAnalysisService/gpas-cli/releases/latest"
+    #     )
+    #     tag = r.json().get("tag_name")
+    #     if tag and tag != __version__:
+    #         logging.warning(
+    #             f"Installed gpas-cli version ({__version__}) differs from the latest release ({tag})"
+    #         )
+    # except:
+    #     pass
+    return user, organisation, allowed_tags, date_mask
 
 
 def update_fasta_header(path: Path, guid: str, name: str):
@@ -336,6 +349,7 @@ class Sample:
         self.decontamination_ref_path = self.get_decontamination_ref_path()
         self.working_dir = Path(working_dir)
         self.working_dir.mkdir(parents=False, exist_ok=True)
+        logging.debug(f"{working_dir=}")
         self.guid = None
         self.mapping_path = None
         self.samtools_path = samtools_path
@@ -367,7 +381,7 @@ class Sample:
             self.fastq = self.working_dir / Path(self.sample_name + ".fastq.gz")
         else:
             cmd = (
-                f'{self.samtools_path} sort "{self.bam}" |'
+                f'{self.samtools_path} sort -n "{self.bam}" |'
                 f" {self.samtools_path} fastq -N"
                 f' -1 "{prefix.parent / (prefix.name + "_1.fastq.gz")}"'
                 f' -2 "{prefix.parent / (prefix.name + "_2.fastq.gz")}"'
@@ -447,26 +461,31 @@ class Batch:
         processes: int = 0,
         environment: ENVIRONMENTS = DEFAULT_ENVIRONMENT,
         json_messages: bool = False,
+        save_reads: bool = False,
     ):
         self.upload_csv = upload_csv
         self.token = parse_token(token) if token else None
         self.environment = environment
         self.working_dir = working_dir
         self.out_dir = out_dir
+        out_dir.mkdir(parents=False, exist_ok=True)
         self.processes = processes if processes else int(multiprocessing.cpu_count())
         self.json = {"validation": "", "decontamination": "", "submission": ""}
         self.json_messages = json_messages
         self.samtools_path = misc.get_binary_path("samtools")
         self.decontaminator_path = misc.get_binary_path("readItAndKeep")
+        self.save_reads = save_reads
         if self.token:
             (
                 self.user,
                 self.organisation,
                 self.permitted_tags,
+                self.date_mask,
             ) = fetch_user_details(self.token["access_token"], self.environment)
             self.headers = {
                 "Authorization": f"Bearer {self.token['access_token']}",
                 "Content-Type": "application/json",
+                "charset": "utf-8",
             }
 
         else:
@@ -474,6 +493,8 @@ class Batch:
             self.organisation = None
             self.permitted_tags = []
             self.headers = None
+            self.date_mask = None
+        logging.debug(f"{self.upload_csv=}")
         self.df, self.schema_name = validate(self.upload_csv, self.permitted_tags)
         self.validation_json = build_validation_message(self.df, self.schema_name)
         batch_attrs = {
@@ -509,6 +530,18 @@ class Batch:
     def _get_decontaminate_cmds(self) -> list[misc.LoggedShellCommand]:
         return [s._get_decontaminate_cmd() for s in self.samples]
 
+    def _save_reads(self) -> None:
+        save_dir = self.out_dir / "decontaminated-reads"
+        save_dir.mkdir(parents=False, exist_ok=True)
+        for s in self.samples:
+            if s.clean_fastq:
+                shutil.copy2(s.clean_fastq, save_dir)
+            if s.clean_fastq1:
+                shutil.copy2(s.clean_fastq1, save_dir)
+            if s.clean_fastq2:
+                shutil.copy2(s.clean_fastq2, save_dir)
+        logging.info(f"Saved decontaminated reads to {save_dir.resolve()}")
+
     def _decontaminate(self) -> None:
         """
         Convert BAM files to FASTQ files, if necessary, and then decontaminate
@@ -520,7 +553,6 @@ class Batch:
                 json_messages=self.json_messages,
                 processes=self.processes,
             )
-
         samples_runs = misc.run_parallel_logged(
             self._get_decontaminate_cmds(),
             participle="Decontaminating",
@@ -528,6 +560,8 @@ class Batch:
             processes=self.processes,
         )
         self._parse_decontamination_stats(samples_runs)
+        if self.save_reads:
+            self._save_reads()
 
     def _parse_decontamination_stats(self, samples_runs) -> None:
         samples_decontamination_stats = {
@@ -550,16 +584,6 @@ class Batch:
             setattr(s, name, value)
 
     def _fetch_guids(self):
-        if not self.headers:
-            logging.warning("No token provided, quitting after decontamination")
-            for s in self.samples:
-                logging.info(
-                    f"{s.sample_name}: {s.clean_fastq}"
-                ) if s.clean_fastq else None
-                logging.info(
-                    f"{s.sample_name}: {s.clean_fastq1} {s.clean_fastq2}"
-                ) if s.clean_fastq1 else None
-            sys.exit()
         md5_attr = "md5" if not self.paired else "md5_1"
         checksums = list(self._get_sample_attrs(md5_attr).values())
         payload = {
@@ -625,7 +649,6 @@ class Batch:
                 "gpas_sample_name",
             ],
         )
-        Path(self.out_dir).mkdir(parents=False, exist_ok=True)
         target_path = self.out_dir / Path(self.batch_guid + ".mapping.csv")
         df.to_csv(target_path, index=False)
         self.mapping_path = target_path
@@ -723,7 +746,11 @@ class Batch:
             + ENDPOINTS[self.environment.value]["ORDS_PATH"]
             + "batches"
         )
-        r = requests.post(url=endpoint, json=self.submission, headers=self.headers)
+        r = requests.post(
+            url=endpoint,
+            data=json.dumps(self.submission, ensure_ascii=False).encode("utf-8"),
+            headers=self.headers,
+        )
         r.raise_for_status()
         logging.debug(f"POSTing JSON {r.text=}")
         if r.json().get("status") != "success":
@@ -758,12 +785,22 @@ class Batch:
 
         samples = []
         for s in self.samples:
+            if self.date_mask in {"WEEK", "MONTH"}:
+                dt = datetime.datetime.strptime(s.collection_date, "%Y-%m-%d")
+                if self.date_mask == "MONTH":
+                    s.collection_date = dt.replace(day=1).strftime("%Y-%m-%d")
+                elif self.date_mask == "WEEK":
+                    td = datetime.timedelta(days=dt.weekday())
+                    s.collection_date = (dt - td).strftime("%Y-%m-%d")
+                logging.info(
+                    f"Masked collection dates to start of {self.date_mask.lower()}"
+                )
             sample = {
                 "name": s.guid,
                 "run_number": s.gpas_run_number,
                 "tags": s.tags,
                 "control": s.control,
-                "collection_date": str(s.collection_date.date()),
+                "collection_date": s.collection_date,
                 "country": s.country,
                 "region": s.region,
                 "district": s.district,
@@ -793,15 +830,19 @@ class Batch:
                 "uploaded_on": str(self.uploaded_on),
                 "uploaded_by": self.user,
                 "organisation": self.organisation,
+                "run_numbers": self.run_numbers,
                 "samples": samples,
             },
         }
         logging.debug(json.dumps(self.submission, indent=4))
 
-    def upload(self, dry_run: bool = False):
+    def upload(self, dry_run: bool = False, save_reads: bool = False) -> None:
         if self.processes > 1:
             logging.info(f"Using {self.processes} processes")
         self._decontaminate()
+        if not self.headers:
+            logging.warning("No token provided, quitting")
+            sys.exit()
         self._hash_fastqs()
         self._fetch_guids()
         self._build_mapping_csv()
@@ -817,13 +858,14 @@ class Batch:
     def _number_runs(self) -> None:
         """Enumerate unique values of run_number for submission"""
         samples_runs = self._get_sample_attrs("run_number")
-        runs = set(samples_runs.values())
-        if list(filter(None, runs)):  # More than just an empty string
-            runs_numbers = {r: str(i) for i, r in enumerate(runs, start=1)}
+        runs = set(list(filter(None, samples_runs.values())))
+        if runs:  # More than just an empty string
+            runs_numbers = {r: i for i, r in enumerate(runs, start=1)}
         else:
-            runs_numbers = {"": ""}
+            runs_numbers = {"": None}
         for s in self.samples:
             s.gpas_run_number = runs_numbers[s.run_number]
+        self.run_numbers = list(filter(None, runs_numbers.values()))
 
 
 def parse_decontamination_stats(stdout: str) -> dict:
