@@ -2,6 +2,7 @@ import hashlib
 import json
 import logging
 import os
+import platform
 import shutil
 import subprocess
 import sys
@@ -192,19 +193,10 @@ def run_parallel_logged(
     if json_messages:
         print_progress_message_json(action=commands[0].action, status="finished")
     else:
-        with logging_redirect_tqdm():
-            logging.info(f"Finished {participle.lower()} {len(commands)} sample(s)")
+        logging.info(f"Finished {participle.lower()} {len(commands)} sample(s)")
     return results
 
 
-def check_unicode(data):
-    """Returns a Unicode object on success or None on failure"""
-    try:
-        return data.decode("utf-8")
-    except UnicodeDecodeError:
-        return None
-
-
 class set_directory(object):
     """
     Context manager for temporarily changing the current working directory
@@ -243,7 +235,10 @@ def get_binary_path(filename: str) -> str:
         path = Path(os.environ[env_var]).resolve()
         logging.debug(f"get_binary_path(): Environment variable mode {path=}")
     elif hasattr(sys, "_MEIPASS"):  # PyInstaller onefile
-        path = (Path(sys.executable).parent / filename).resolve()
+        if platform.system() == "Windows":
+            path = (Path(sys.executable).parent / (filename + ".exe")).resolve()
+        else:
+            path = (Path(sys.executable).parent / filename).resolve()
         logging.debug(f"get_binary_path(): PyInstaller mode {path=}")
     elif shutil.which(filename):  # $PATH
         path = Path(shutil.which(filename)).resolve()
